# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

import tempfile
from unittest import TestCase
from conda.testing.integration import run_command, Commands

import pytest

from conda.models.match_spec import MatchSpec
from conda.exceptions import UnsatisfiableError
from conda.gateways.disk.delete import rm_rf

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


@pytest.fixture
def fix_cli_install(tmpdir):
    prefix = tmpdir.mkdir("cli_install_prefix")
    test_env = tmpdir.mkdir("cli_install_test_env")
    run_command(Commands.CREATE, str(prefix), 'python=3.7')
    yield str(prefix), str(test_env)
    rm_rf(prefix)
    rm_rf(test_env)


@pytest.mark.integration
def test_pre_link_message(fix_cli_install, conda_build_recipes):
    prefix = fix_cli_install[0]
    with patch("conda.cli.common.confirm_yn", return_value=True):
        stdout, _, _ = run_command(
            Commands.INSTALL, prefix, "pre_link_messages_package", "--use-local"
        )
        assert "Lorem ipsum dolor sit amet" in stdout


@pytest.mark.integration
def test_find_conflicts_called_once(fix_cli_install):
    prefix, test_env = fix_cli_install
    bad_deps = {'python': {((MatchSpec("statistics"), MatchSpec("python[version='>=2.7,<2.8.0a0']")), 'python=3')}}

    with patch(
            "conda.resolve.Resolve.find_conflicts",
            side_effect=UnsatisfiableError(bad_deps, strict=True),
        ) as mocked_find_conflicts:
            with pytest.raises(UnsatisfiableError):
                # Statistics is a py27 only package allowing us a simple unsatisfiable case
                run_command(Commands.INSTALL, prefix, "statistics")
            assert mocked_find_conflicts.call_count == 1

            mocked_find_conflicts.reset_mock()
            with pytest.raises(UnsatisfiableError):
                run_command(Commands.INSTALL, prefix, "statistics", "--freeze-installed")
            assert mocked_find_conflicts.call_count == 1

            mocked_find_conflicts.reset_mock()
            with pytest.raises(UnsatisfiableError):
                run_command(Commands.CREATE, test_env, "statistics", "python=3.7")
            assert mocked_find_conflicts.call_count == 1



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

import unittest

from .. import support_file

from conda_env import env
from conda_env.specs.requirements import RequirementsSpec


class TestRequiremets(unittest.TestCase):
    def test_no_environment_file(self):
        spec = RequirementsSpec(name=None, filename='not-a-file')
        self.assertEqual(spec.can_handle(), False)

    def test_no_name(self):
        spec = RequirementsSpec(filename=support_file('requirements.txt'))
        self.assertEqual(spec.can_handle(), False)

    def test_req_file_and_name(self):
        spec = RequirementsSpec(filename=support_file('requirements.txt'), name='env')
        self.assertTrue(spec.can_handle())

    def test_environment(self):
        spec = RequirementsSpec(filename=support_file('requirements.txt'), name='env')
        self.assertIsInstance(spec.environment, env.Environment)
        self.assertEqual(
            spec.environment.dependencies['conda'][0],
            'flask==0.10.1'
        )



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

from conda.core.solve import Solver
from conda.testing.solver_helpers import SolverTests


class TestClassicSolver(SolverTests):
    @property
    def solver_class(self):
        return Solver



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause
r"""
Example packages used in this data testing:
- https://github.com/cherrypy/cherrypy.git
- https://github.com/dask/dask.git
- https://github.com/rdegges/django-twilio.git
- https://github.com/frejanordsiek/hdf5storage.git
- https://github.com/jaraco/keyring.git
- https://github.com/scrapy/scrapy.git
- https://github.com/tox-dev/tox.git
- https://github.com/twilio/twilio-python.git


conda create -y -p ./tenv python=2 numpy h5py nomkl lxml
conda activate ./tenv
git clone https://github.com/frejanordsiek/hdf5storage.git && pushd hdf5storage && git checkout 0.1.15 && pip install . --no-binary :all: && popd && rm -rf hdf5storage
git clone https://github.com/cherrypy/cherrypy.git && pushd cherrypy && git checkout v17.2.0 && pip install . --no-binary :all: && popd && rm -rf cherrypy
git clone https://github.com/dask/dask.git && pushd dask && git checkout 0.18.2 && pip install . --no-binary :all: && popd && rm -rf dask
git clone https://github.com/rdegges/django-twilio.git && pushd django-twilio && git checkout 0.9.2 && pip install . --no-binary :all: && popd && rm -rf django-twilio
git clone https://github.com/jaraco/keyring.git && pushd keyring && git checkout 13.2.1 && pip install . --no-binary :all: && popd && rm -rf keyring
git clone https://github.com/scrapy/scrapy.git && pushd scrapy && git checkout 1.5.1 && pip install . --no-binary :all: && popd && rm -rf scrapy
git clone https://github.com/tox-dev/tox.git && pushd tox && git checkout 3.2.1 && pip install . --no-binary :all: && popd && rm -rf tox
git clone https://github.com/twilio/twilio-python.git && pushd twilio-python && git checkout 6.16.1 && pip install . --no-binary :all: && popd && rm -rf twilio-python
conda deactivate



conda create -y -p ./tenv python=2 numpy h5py nomkl lxml pip=8
./tenv/bin/pip install --no-binary :all: twilio==6.16.1
./tenv/bin/pip install --no-binary :all: hdf5storage==0.1.15
./tenv/bin/pip install --no-binary :all: cherrypy==17.2.0
./tenv/bin/pip install --no-binary :all: dask==0.18.2
./tenv/bin/pip install --no-binary :all: django-twilio==0.9.2
./tenv/bin/pip install --no-binary :all: keyring==13.2.1
./tenv/bin/pip install --no-binary :all: scrapy==1.5.1
./tenv/bin/pip install --no-binary :all: tox==3.2.1
./tenv/bin/pip install --no-binary :all: backports.weakref
./tenv/bin/pip install --no-binary :all: backports.pdb
./tenv/bin/pip install --no-binary :all: backports.shutil_which

mkdir -p ./py27-osx-no-binary/lib/python2.7
mv ./tenv/lib/python2.7/site-packages ./py27-osx-no-binary/lib/python2.7/
mv ./tenv/conda-meta ./py27-osx-no-binary/
rm -rf ./tenv
find ./py27-osx-no-binary \( -name \*.py -o -name \*.py[cod] -o -name __pycache__ \) -delete
find ./py27-osx-no-binary \( -name \*.so -o -name \*.h -o -name \*.c -o -name \*.mo -o -name \*.po -o -name \*.cfg \) -delete
find ./py27-osx-no-binary \( -name \*.pxd -o -name \*.dat -o -name \*.pem -o -name \*.rng -o -name \*.xsl \) -delete
find ./py27-osx-no-binary -type d \( -name django -o -name zoneinfo -o -name twisted -o -name cherrypy -o -name scrapy -o -name numpy \) | xargs rm -rf
find ./py27-osx-no-binary -type d \( -name setuptools -o -name lxml -o -name virtualenv_support -o -name pip -o -name enum \) | xargs rm -rf
# find ./py27-osx-no-binary \( -name SOURCES.txt \) -delete
find ./py27-osx-no-binary -type d | gtac | xargs rmdir || true




conda create -y -p ./tenv python=3.6 numpy h5py nomkl lxml
./tenv/bin/pip install twilio==6.16.1
./tenv/bin/pip install hdf5storage==0.1.15
./tenv/bin/pip install cherrypy==17.2.0
./tenv/bin/pip install dask==0.18.2
./tenv/bin/pip install django-twilio==0.9.2
./tenv/bin/pip install keyring==13.2.1
./tenv/bin/pip install scrapy==1.5.1
./tenv/bin/pip install tox==3.2.1

mkdir -p ./py36-osx-whl/lib/python3.6
mv ./tenv/lib/python3.6/site-packages ./py36-osx-whl/lib/python3.6/
mv ./tenv/conda-meta ./py36-osx-whl/
rm -rf ./tenv
find ./py36-osx-whl \( -name \*.py -o -name \*.py[cod] -o -name __pycache__ \) -delete
find ./py36-osx-whl \( -name \*.so -o -name \*.h -o -name \*.c -o -name \*.mo -o -name \*.po -o -name \*.cfg \) -delete
find ./py36-osx-whl \( -name \*.pxd -o -name \*.dat -o -name \*.pem -o -name \*.rng -o -name \*.xsl \) -delete
find ./py36-osx-whl -type d \( -name django -o -name zoneinfo -o -name twisted -o -name cherrypy -o -name scrapy -o -name numpy \) | xargs rm -rf
find ./py36-osx-whl -type d \( -name setuptools -o -name lxml -o -name virtualenv_support -o -name pip -o -name enum \) | xargs rm -rf
find ./py36-osx-whl -type d | gtac | xargs rmdir || true




"""

import os

HERE = os.path.abspath(os.path.dirname(__file__))

# Test environment installed using either `pip install <pth-to-wheel>` or
# `python setup.py install`
PATH_TEST_ENV_1 = os.path.join(HERE, 'envpy27osx')
PATH_TEST_ENV_2 = os.path.join(HERE, 'envpy37osx_whl')
PATH_TEST_ENV_3 = os.path.join(HERE, 'envpy37win')
PATH_TEST_ENV_4 = os.path.join(HERE, 'envpy27win_whl')

METADATA_241_PATH = os.path.join(HERE, 'pep241', 'PKG-INFO')
METADATA_314_PATH = os.path.join(HERE, 'pep314', 'PKG-INFO')
METADATA_345_PATH = os.path.join(HERE, 'pep345', 'PKG-INFO')
METADATA_566_PATH = os.path.join(HERE, 'pep566', 'PKG-INFO')

METADATA_VERSION_PATHS = (METADATA_241_PATH, METADATA_314_PATH,
                          METADATA_345_PATH, METADATA_566_PATH)



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import absolute_import, division, print_function, unicode_literals

from conda.core.portability import SHEBANG_REGEX, replace_long_shebang
from conda.models.enums import FileMode
from logging import getLogger
import re
from unittest import TestCase

log = getLogger(__name__)


class ReplaceShebangTests(TestCase):

    def test_shebang_regex_matches(self):
        shebang = b"#!/simple/shebang"
        match = re.match(SHEBANG_REGEX, shebang, re.MULTILINE)
        assert match.groups() == (b'#!/simple/shebang',
                                  b'/simple/shebang',
                                  b'')

        # two lines
        shebang = b"#!/simple/shebang\nsecond line\n"
        match = re.match(SHEBANG_REGEX, shebang, re.MULTILINE)
        assert match.groups() == (b'#!/simple/shebang',
                                  b'/simple/shebang',
                                  b'')

        # with spaces
        shebang = b"#!/simple/shebang\nsecond line\n"
        match = re.match(SHEBANG_REGEX, shebang, re.MULTILINE)
        assert match.groups() == (b'#!/simple/shebang',
                                  b'/simple/shebang',
                                  b'')

        # with spaces
        shebang = b"#!    /simple/shebang\nsecond line\n"
        match = re.match(SHEBANG_REGEX, shebang, re.MULTILINE)
        assert match.groups() == (b'#!    /simple/shebang',
                                  b'/simple/shebang',
                                  b'')

        # with escaped spaces and flags
        shebang = b"#!/simple/shebang/escaped\\ space --and --flags -x\nsecond line\n"
        match = re.match(SHEBANG_REGEX, shebang, re.MULTILINE)
        assert match.groups() == (b'#!/simple/shebang/escaped\\ space --and --flags -x',
                                  b'/simple/shebang/escaped\\ space',
                                  b' --and --flags -x')


    def test_replace_long_shebang(self):
        content_line = b"content line " * 5

        # simple shebang no replacement
        shebang = b"#!/simple/shebang/escaped\\ space --and --flags -x"
        data = b'\n'.join((shebang, content_line, content_line, content_line))
        new_data = replace_long_shebang(FileMode.text, data)
        assert data == new_data

        # long shebang with truncation
        #   executable name is 'escaped space'
        shebang = b"#!/" + b"shebang/" * 20 + b"python" + b" --and --flags -x"
        assert len(shebang) > 127
        data = b'\n'.join((shebang, content_line, content_line, content_line))
        new_data = replace_long_shebang(FileMode.text, data)
        new_shebang = b"#!/usr/bin/env python --and --flags -x"
        new_expected_data = b'\n'.join((new_shebang, content_line, content_line, content_line))
        assert new_expected_data == new_data

        # long shebang with truncation
        #   executable name is 'escaped space'
        shebang = b"#!/" + b"shebang/" * 20 + b"escaped\\ space" + b" --and --flags -x"
        assert len(shebang) > 127
        data = b'\n'.join((shebang, content_line, content_line, content_line))
        new_data = replace_long_shebang(FileMode.text, data)
        new_shebang = b"#!/usr/bin/env escaped\\ space --and --flags -x"
        new_expected_data = b'\n'.join((new_shebang, content_line, content_line, content_line))
        assert new_expected_data == new_data



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

from argparse import ArgumentParser
from contextlib import contextmanager
from tempfile import mkdtemp

from conda.gateways.disk.delete import rm_rf

from conda_env.cli.main import do_call as do_call_conda_env
from conda_env.cli.main_create import configure_parser as create_configure_parser
from conda_env.cli.main_update import configure_parser as update_configure_parser
from conda_env.cli.main_export import configure_parser as export_configure_parser

from conda.utils import massage_arguments

class Commands:
    CREATE = "create"
    UPDATE = "update"
    EXPORT = "export"


parser_config = {
    Commands.CREATE: create_configure_parser,
    Commands.UPDATE: update_configure_parser,
    Commands.EXPORT: export_configure_parser,
}


@contextmanager
def make_temp_envs_dir():
    envs_dir = mkdtemp()
    try:
        yield envs_dir
    finally:
        rm_rf(envs_dir)


def run_command(command, env_name, *arguments):
    arguments = massage_arguments(arguments)
    args = [command, "-n", env_name, "-f"] + arguments

    p = ArgumentParser()
    sub_parsers = p.add_subparsers(metavar='command', dest='cmd')
    parser_config[command](sub_parsers)
    args = p.parse_args(args)

    do_call_conda_env(args, p)



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

from setuptools import setup

setup(
    name='module_to_install_in_editable_mode',
    packages=[],
)



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause
import os
from os.path import isdir, join
import sys

from conda.auxlib.entity import EntityEncoder
from conda.auxlib.path import expand
from conda.base.context import context
from conda.cli import install as cli_install
from conda.cli import common as cli_common
from conda.gateways.connection.session import CONDA_SESSION_SCHEMES

base_env_name = 'base'


def stdout_json(d):
    import json

    json.dump(d, sys.stdout, indent=2, sort_keys=True, cls=EntityEncoder)
    sys.stdout.write('\n')


def get_prefix(args, search=True):
    from conda.base.context import determine_target_prefix
    return determine_target_prefix(context, args)


def find_prefix_name(name):
    if name == base_env_name:
        return context.root_prefix
    # always search cwd in addition to envs dirs (for relative path access)
    for envs_dir in list(context.envs_dirs) + [os.getcwd(), ]:
        prefix = join(envs_dir, name)
        if isdir(prefix):
            return prefix
    return None


def print_result(args, prefix, result):
    if context.json:
        if result["conda"] is None and result["pip"] is None:
            cli_common.stdout_json_success(message='All requested packages already installed.')
        else:
            if result["conda"] is not None:
                actions = result["conda"]
            else:
                actions = {}
            if result["pip"] is not None:
                actions["PIP"] = result["pip"]
            cli_common.stdout_json_success(prefix=prefix, actions=actions)
    else:
        cli_install.print_activate(args.name if args.name else prefix)


def get_filename(filename):
    """Expand filename if local path or return the url"""
    url_scheme = filename.split("://", 1)[0]
    if url_scheme in CONDA_SESSION_SCHEMES:
        return filename
    else:
        return expand(filename)



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause
from __future__ import absolute_import, division, print_function, unicode_literals

import hashlib
import json
import os
from os.path import abspath, basename, dirname, isdir, isfile, islink, join
import re
import tarfile
import tempfile

from ..auxlib.entity import EntityEncoder
from ..base.constants import CONDA_PACKAGE_EXTENSION_V1
from ..base.context import context
from ..common.compat import PY3
from ..common.path import paths_equal
from ..core.prefix_data import PrefixData
from ..gateways.disk.delete import rmtree
from ..install import PREFIX_PLACEHOLDER
from ..misc import untracked


def remove(prefix, files):
    """
    Remove files for a given prefix.
    """
    dst_dirs = set()
    for f in files:
        dst = join(prefix, f)
        dst_dirs.add(dirname(dst))
        os.unlink(dst)

    for path in sorted(dst_dirs, key=len, reverse=True):
        try:
            os.rmdir(path)
        except OSError:  # directory might not be empty
            pass


def execute(args, parser):

    prefix = context.target_prefix

    if args.which:
        for path in args.which:
            for prec in which_package(path):
                print('%-50s  %s' % (path, prec.dist_str()))
        return

    print('# prefix:', prefix)

    if args.reset:
        remove(prefix, untracked(prefix))
        return

    if args.untracked:
        files = sorted(untracked(prefix))
        print('# untracked files: %d' % len(files))
        for fn in files:
            print(fn)
        return

    make_tarbz2(prefix,
                name=args.pkg_name.lower(),
                version=args.pkg_version,
                build_number=int(args.pkg_build))


def get_installed_version(prefix, name):
    for info in PrefixData(prefix).iter_records():
        if info['name'] == name:
            return str(info['version'])
    return None


def create_info(name, version, build_number, requires_py):
    d = dict(
        name=name,
        version=version,
        platform=context.platform,
        arch=context.arch_name,
        build_number=int(build_number),
        build=str(build_number),
        depends=[],
    )
    if requires_py:
        d['build'] = ('py%d%d_' % requires_py) + d['build']
        d['depends'].append('python %d.%d*' % requires_py)
    return d


shebang_pat = re.compile(r'^#!.+$', re.M)
def fix_shebang(tmp_dir, path):
    if open(path, 'rb').read(2) != '#!':
        return False

    with open(path) as fi:
        data = fi.read()
    m = shebang_pat.match(data)
    if not (m and 'python' in m.group()):
        return False

    data = shebang_pat.sub('#!%s/bin/python' % PREFIX_PLACEHOLDER,
                           data, count=1)
    tmp_path = join(tmp_dir, basename(path))
    with open(tmp_path, 'w') as fo:
        fo.write(data)
    os.chmod(tmp_path, int('755', 8))
    return True


def _add_info_dir(t, tmp_dir, files, has_prefix, info):
    info_dir = join(tmp_dir, 'info')
    os.mkdir(info_dir)
    with open(join(info_dir, 'files'), 'w') as fo:
        for f in files:
            fo.write(f + '\n')

    with open(join(info_dir, 'index.json'), 'w') as fo:
        json.dump(info, fo, indent=2, sort_keys=True, cls=EntityEncoder)

    if has_prefix:
        with open(join(info_dir, 'has_prefix'), 'w') as fo:
            for f in has_prefix:
                fo.write(f + '\n')

    for fn in os.listdir(info_dir):
        t.add(join(info_dir, fn), 'info/' + fn)


def create_conda_pkg(prefix, files, info, tar_path, update_info=None):
    """
    create a conda package with `files` (in `prefix` and `info` metadata)
    at `tar_path`, and return a list of warning strings
    """
    files = sorted(files)
    warnings = []
    has_prefix = []
    tmp_dir = tempfile.mkdtemp()
    t = tarfile.open(tar_path, 'w:bz2')
    h = hashlib.new('sha1')
    for f in files:
        assert not (f.startswith('/') or f.endswith('/') or '\\' in f or f == ''), f
        path = join(prefix, f)
        if f.startswith('bin/') and fix_shebang(tmp_dir, path):
            path = join(tmp_dir, basename(path))
            has_prefix.append(f)
        t.add(path, f)
        h.update(f.encode('utf-8'))
        h.update(b'\x00')
        if islink(path):
            link = os.readlink(path)
            if PY3 and isinstance(link, str):
                h.update(bytes(link, 'utf-8'))
            else:
                h.update(link)
            if link.startswith('/'):
                warnings.append('found symlink to absolute path: %s -> %s' %
                                (f, link))
        elif isfile(path):
            h.update(open(path, 'rb').read())
            if path.endswith('.egg-link'):
                warnings.append('found egg link: %s' % f)

    info['file_hash'] = h.hexdigest()
    if update_info:
        update_info(info)
    _add_info_dir(t, tmp_dir, files, has_prefix, info)
    t.close()
    rmtree(tmp_dir)
    return warnings


def make_tarbz2(prefix, name='unknown', version='0.0', build_number=0,
                files=None):
    if files is None:
        files = untracked(prefix)
    print("# files: %d" % len(files))
    if len(files) == 0:
        print("# failed: nothing to do")
        return None

    if any('/site-packages/' in f for f in files):
        python_version = get_installed_version(prefix, 'python')
        assert python_version is not None
        requires_py = tuple(int(x) for x in python_version[:3].split('.'))
    else:
        requires_py = False

    info = create_info(name, version, build_number, requires_py)
    tarbz2_fn = ('%(name)s-%(version)s-%(build)s' % info) + CONDA_PACKAGE_EXTENSION_V1
    create_conda_pkg(prefix, files, info, tarbz2_fn)
    print('# success')
    print(tarbz2_fn)
    return tarbz2_fn


def which_package(path):
    """
    given the path (of a (presumably) conda installed file) iterate over
    the conda packages the file came from.  Usually the iteration yields
    only one package.
    """
    path = abspath(path)
    prefix = which_prefix(path)
    if prefix is None:
        from ..exceptions import CondaVerificationError
        raise CondaVerificationError("could not determine conda prefix from: %s" % path)

    for prec in PrefixData(prefix).iter_records():
        if any(paths_equal(join(prefix, f), path) for f in prec['files'] or ()):
            yield prec


def which_prefix(path):
    """
    given the path (to a (presumably) conda installed file) return the
    environment prefix in which the file in located
    """
    prefix = abspath(path)
    while True:
        if isdir(join(prefix, 'conda-meta')):
            # we found the it, so let's return it
            return prefix
        if prefix == dirname(prefix):
            # we cannot chop off any more directories, so we didn't find it
            return None
        prefix = dirname(prefix)



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import absolute_import, division, print_function, unicode_literals

from itertools import chain
import os
from os.path import join, abspath
from tempfile import gettempdir
from unittest import TestCase, mock

import pytest

from conda.auxlib.collection import AttrDict
from conda.auxlib.ish import dals
from conda._vendor.toolz.itertoolz import concat
from conda.base.constants import PathConflict, ChannelPriority
from conda.base.context import context, reset_context, conda_tests_ctxt_mgmt_def_pol
from conda.common.compat import odict, iteritems
from conda.common.configuration import ValidationError, YamlRawParameter
from conda.common.io import env_var, env_vars
from conda.common.path import expand, win_path_backout
from conda.common.url import join_url, path_to_url
from conda.common.serialize import yaml_round_trip_load
from conda.core.package_cache_data import PackageCacheData
from conda.gateways.disk.create import mkdir_p, create_package_cache_directory
from conda.gateways.disk.delete import rm_rf
from conda.gateways.disk.permissions import make_read_only
from conda.gateways.disk.update import touch
from conda.models.channel import Channel
from conda.models.match_spec import MatchSpec
from conda.utils import on_win

from conda.testing.helpers import tempdir


class ContextCustomRcTests(TestCase):

    def setUp(self):
        string = dals("""
        custom_channels:
          darwin: https://some.url.somewhere/stuff
          chuck: http://another.url:8080/with/path
        custom_multichannels:
          michele:
            - https://do.it.with/passion
            - learn_from_every_thing
          steve:
            - more-downloads
        migrated_custom_channels:
          darwin: s3://just/cant
          chuck: file:///var/lib/repo/
        migrated_channel_aliases:
          - https://conda.anaconda.org
        channel_alias: ftp://new.url:8082
        conda-build:
          root-dir: /some/test/path
        proxy_servers:
          http: http://user:pass@corp.com:8080
          https: none
          ftp:
          sftp: ''
          ftps: false
          rsync: 'false'
        aggressive_update_packages: []
        channel_priority: false
        """)
        reset_context(())
        rd = odict(testdata=YamlRawParameter.make_raw_parameters('testdata', yaml_round_trip_load(string)))
        context._set_raw_data(rd)

    def tearDown(self):
        reset_context()

    def test_migrated_custom_channels(self):
        assert Channel('https://some.url.somewhere/stuff/darwin/noarch/a-mighty-fine.tar.bz2').canonical_name == 'darwin'
        assert Channel('s3://just/cant/darwin/noarch/a-mighty-fine.tar.bz2').canonical_name == 'darwin'
        assert Channel('s3://just/cant/darwin/noarch/a-mighty-fine.tar.bz2').urls() == [
            'https://some.url.somewhere/stuff/darwin/noarch']

    def test_old_channel_alias(self):
        platform = context.subdir

        cf_urls = ["ftp://new.url:8082/conda-forge/%s" % platform,
                   "ftp://new.url:8082/conda-forge/noarch"]
        assert Channel('conda-forge').urls() == cf_urls

        url = "https://conda.anaconda.org/conda-forge/osx-64/some-great-package.tar.bz2"
        assert Channel(url).canonical_name == 'conda-forge'
        assert Channel(url).base_url == 'ftp://new.url:8082/conda-forge'
        assert Channel(url).urls() == [
            'ftp://new.url:8082/conda-forge/osx-64',
            'ftp://new.url:8082/conda-forge/noarch'
        ]
        assert Channel("https://conda.anaconda.org/conda-forge/label/dev/linux-64/"
                       "some-great-package.tar.bz2").urls() == [
            "ftp://new.url:8082/conda-forge/label/dev/linux-64",
            "ftp://new.url:8082/conda-forge/label/dev/noarch",
        ]

    def test_signing_metadata_url_base(self):
        SIGNING_URL_BASE = "https://conda.example.com/pkgs"
        string = f"signing_metadata_url_base: {SIGNING_URL_BASE}"
        reset_context()
        rd = odict(testdata=YamlRawParameter.make_raw_parameters('testdata', yaml_round_trip_load(string)))
        context._set_raw_data(rd)
        assert context.signing_metadata_url_base == SIGNING_URL_BASE

    def test_signing_metadata_url_base_empty_default_channels(self):
        string = dals("""
        default_channels: []
        """)
        reset_context()
        rd = odict(testdata=YamlRawParameter.make_raw_parameters('testdata', yaml_round_trip_load(string)))
        context._set_raw_data(rd)
        assert len(context.default_channels) is 0
        assert context.signing_metadata_url_base is None


    def test_client_ssl_cert(self):
        string = dals("""
        client_ssl_cert_key: /some/key/path
        """)
        reset_context()
        rd = odict(testdata=YamlRawParameter.make_raw_parameters('testdata', yaml_round_trip_load(string)))
        context._set_raw_data(rd)
        pytest.raises(ValidationError, context.validate_configuration)

    def test_conda_envs_path(self):
        saved_envs_path = os.environ.get('CONDA_ENVS_PATH')
        beginning = "C:" + os.sep if on_win else os.sep
        path1 = beginning + os.sep.join(['my', 'envs', 'dir', '1'])
        path2 = beginning + os.sep.join(['my', 'envs', 'dir', '2'])
        try:
            os.environ['CONDA_ENVS_PATH'] = path1
            reset_context()
            assert context.envs_dirs[0] == path1

            os.environ['CONDA_ENVS_PATH'] = os.pathsep.join([path1, path2])
            reset_context()
            assert context.envs_dirs[0] == path1
            assert context.envs_dirs[1] == path2
        finally:
            if saved_envs_path:
                os.environ['CONDA_ENVS_PATH'] = saved_envs_path
            else:
                del os.environ['CONDA_ENVS_PATH']

    def test_conda_bld_path(self):
        conda_bld_path = join(gettempdir(), 'conda-bld')
        conda_bld_url = path_to_url(conda_bld_path)
        try:
            mkdir_p(conda_bld_path)
            with env_var('CONDA_BLD_PATH', conda_bld_path, stack_callback=conda_tests_ctxt_mgmt_def_pol):
                assert len(context.conda_build_local_paths) >= 1
                assert context.conda_build_local_paths[0] == conda_bld_path

                channel = Channel('local')
                assert channel.channel_name == "local"
                assert channel.channel_location is None
                assert channel.platform is None
                assert channel.package_filename is None
                assert channel.auth is None
                assert channel.token is None
                assert channel.scheme is None
                assert channel.canonical_name == "local"
                assert channel.url() is None
                urls = list(concat((
                               join_url(url, context.subdir),
                               join_url(url, 'noarch'),
                           ) for url in context.conda_build_local_urls))
                assert channel.urls() == urls

                channel = Channel(conda_bld_url)
                assert channel.canonical_name == "local"
                assert channel.platform is None
                assert channel.package_filename is None
                assert channel.auth is None
                assert channel.token is None
                assert channel.scheme == "file"
                assert channel.urls() == [
                    join_url(conda_bld_url, context.subdir),
                    join_url(conda_bld_url, 'noarch'),
                ]
                assert channel.url() == join_url(conda_bld_url, context.subdir)
                assert channel.channel_name.lower() == win_path_backout(conda_bld_path).lstrip('/').lower()
                assert channel.channel_location == ''  # location really is an empty string; all path information is in channel_name
                assert channel.canonical_name == "local"
        finally:
            rm_rf(conda_bld_path)

    def test_custom_multichannels(self):
        assert context.custom_multichannels['michele'] == (
            Channel('passion'),
            Channel('learn_from_every_thing'),
        )

    def test_restore_free_channel(self):
        assert 'https://repo.anaconda.com/pkgs/free' not in context.default_channels
        with env_var("CONDA_RESTORE_FREE_CHANNEL", 'true', stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert  context.default_channels.index('https://repo.anaconda.com/pkgs/free') == 1

    def test_proxy_servers(self):
        assert context.proxy_servers['http'] == 'http://user:pass@corp.com:8080'
        assert context.proxy_servers['https'] is None
        assert context.proxy_servers['ftp'] is None
        assert context.proxy_servers['sftp'] == ''
        assert context.proxy_servers['ftps'] == 'False'
        assert context.proxy_servers['rsync'] == 'false'

    def test_conda_build_root_dir(self):
        assert context.conda_build['root-dir'] == "/some/test/path"

    def test_clobber_enum(self):
        with env_var("CONDA_PATH_CONFLICT", 'prevent', stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert context.path_conflict == PathConflict.prevent

    def test_context_parameter_map(self):
        all_parameter_names = context.list_parameters()
        all_mapped_parameter_names = tuple(chain.from_iterable(context.category_map.values()))

        unmapped_parameter_names = set(all_parameter_names) - set(all_mapped_parameter_names)
        assert not unmapped_parameter_names, unmapped_parameter_names

        assert len(all_parameter_names) == len(all_mapped_parameter_names)

    def test_context_parameters_have_descriptions(self):
        skip_categories = ('CLI-only', 'Hidden and Undocumented')
        documented_parameter_names = chain.from_iterable((
            parameter_names for category, parameter_names in iteritems(context.category_map)
            if category not in skip_categories
        ))

        from pprint import pprint
        for name in documented_parameter_names:
            description = context.get_descriptions()[name]
            pprint(context.describe_parameter(name))

    def test_local_build_root_custom_rc(self):
        assert context.local_build_root == abspath("/some/test/path")

        test_path_1 = join(os.getcwd(), 'test_path_1')
        with env_var("CONDA_CROOT", test_path_1, stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert context.local_build_root == test_path_1

        test_path_2 = join(os.getcwd(), 'test_path_2')
        with env_var("CONDA_BLD_PATH", test_path_2, stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert context.local_build_root == test_path_2

    def test_default_target_is_root_prefix(self):
        assert context.target_prefix == context.root_prefix

    def test_target_prefix(self):
        with tempdir() as prefix:
            mkdir_p(join(prefix, 'first', 'envs'))
            mkdir_p(join(prefix, 'second', 'envs'))
            create_package_cache_directory(join(prefix, 'first', 'pkgs'))
            create_package_cache_directory(join(prefix, 'second', 'pkgs'))
            envs_dirs = (join(prefix, 'first', 'envs'), join(prefix, 'second', 'envs'))
            with env_var('CONDA_ENVS_DIRS', os.pathsep.join(envs_dirs), stack_callback=conda_tests_ctxt_mgmt_def_pol):

                # with both dirs writable, choose first
                reset_context((), argparse_args=AttrDict(name='blarg', func='create'))
                assert context.target_prefix == join(envs_dirs[0], 'blarg')

                # with first dir read-only, choose second
                PackageCacheData._cache_.clear()
                make_read_only(join(envs_dirs[0], '.conda_envs_dir_test'))
                reset_context((), argparse_args=AttrDict(name='blarg', func='create'))
                assert context.target_prefix == join(envs_dirs[1], 'blarg')

                # if first dir is read-only but environment exists, choose first
                PackageCacheData._cache_.clear()
                mkdir_p(join(envs_dirs[0], 'blarg'))
                touch(join(envs_dirs[0], 'blarg', 'history'))
                reset_context((), argparse_args=AttrDict(name='blarg', func='create'))
                assert context.target_prefix == join(envs_dirs[0], 'blarg')

    def test_aggressive_update_packages(self):
        assert context.aggressive_update_packages == tuple()
        specs = ['certifi', 'openssl>=1.1']
        with env_var('CONDA_AGGRESSIVE_UPDATE_PACKAGES', ','.join(specs), stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert context.aggressive_update_packages == tuple(MatchSpec(s) for s in specs)

    def test_channel_priority(self):
        assert context.channel_priority == ChannelPriority.DISABLED

    def test_cuda_detection(self):
        # confirm that CUDA detection doesn't raise exception
        version = context.cuda_version
        assert version is None or isinstance(version, str)

    def test_cuda_override(self):
        with env_var('CONDA_OVERRIDE_CUDA', '4.5'):
            version = context.cuda_version
            assert version == '4.5'

    def test_cuda_override_none(self):
        with env_var('CONDA_OVERRIDE_CUDA', ''):
            version = context.cuda_version
            assert version is None

    def test_threads(self):
        default_value = None
        assert context.default_threads == default_value
        assert context.repodata_threads == default_value
        assert context.verify_threads == 1
        assert context.execute_threads == 1

        with env_var('CONDA_DEFAULT_THREADS', '3',
                     stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert context.default_threads == 3
            assert context.verify_threads == 3
            assert context.repodata_threads == 3
            assert context.execute_threads == 3

        with env_var('CONDA_VERIFY_THREADS', '3',
                     stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert context.default_threads == default_value
            assert context.verify_threads == 3
            assert context.repodata_threads == default_value
            assert context.execute_threads == 1

        with env_var('CONDA_REPODATA_THREADS', '3',
                     stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert context.default_threads == default_value
            assert context.verify_threads == 1
            assert context.repodata_threads == 3
            assert context.execute_threads == 1

        with env_var('CONDA_EXECUTE_THREADS', '3',
                     stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert context.default_threads == default_value
            assert context.verify_threads == 1
            assert context.repodata_threads == default_value
            assert context.execute_threads == 3

        with env_vars({'CONDA_EXECUTE_THREADS': '3',
                       'CONDA_DEFAULT_THREADS': '1'},
                      stack_callback=conda_tests_ctxt_mgmt_def_pol):

            assert context.default_threads == 1
            assert context.verify_threads == 1
            assert context.repodata_threads == 1
            assert context.execute_threads == 3

    def test_channels_defaults(self):
        """
        Test when no channels provided in cli
        """
        reset_context(())
        assert context.channels == ('defaults',)

    def test_channels_defaults_condarc(self):
        """
        Test when no channels provided in cli, but some in condarc
        """
        reset_context(())
        string = dals("""
        channels: ['defaults', 'conda-forge']
        """)
        rd = odict(testdata=YamlRawParameter.make_raw_parameters('testdata', yaml_round_trip_load(string)))
        context._set_raw_data(rd)
        assert context.channels == ('defaults', 'conda-forge')

    def test_specify_channels_cli_adding_defaults_no_condarc(self):
        """
        When the channel haven't been specified in condarc, 'defaults'
        should be present when specifying channel in the cli
        """
        reset_context((), argparse_args=AttrDict(channel=['conda-forge']))
        assert context.channels == ('conda-forge', 'defaults')

    def test_specify_channels_cli_condarc(self):
        """
        When the channel have been specified in condarc, these channels
        should be used along with the one specified
        """
        reset_context((), argparse_args=AttrDict(channel=['conda-forge']))
        string = dals("""
        channels: ['defaults', 'conda-forge']
        """)
        rd = odict(testdata=YamlRawParameter.make_raw_parameters('testdata', yaml_round_trip_load(string)))
        context._set_raw_data(rd)
        assert context.channels == ('defaults', 'conda-forge')

    def test_specify_different_channels_cli_condarc(self):
        """
        When the channel have been specified in condarc, these channels
        should be used along with the one specified
        In this test, the given channel in cli is different from condarc
        'defaults' should not be added
        """
        reset_context((), argparse_args=AttrDict(channel=['other']))
        string = dals("""
        channels: ['conda-forge']
        """)
        rd = odict(testdata=YamlRawParameter.make_raw_parameters('testdata', yaml_round_trip_load(string)))
        context._set_raw_data(rd)
        assert context.channels == ('conda-forge', 'other')

    def test_specify_same_channels_cli_as_in_condarc(self):
        """
        When the channel have been specified in condarc, these channels
        should be used along with the one specified

        In this test, the given channel in cli is the same as in condarc
        'defaults' should not be added
        See https://github.com/conda/conda/issues/10732
        """
        reset_context((), argparse_args=AttrDict(channel=['conda-forge']))
        string = dals("""
        channels: ['conda-forge']
        """)
        rd = odict(testdata=YamlRawParameter.make_raw_parameters('testdata', yaml_round_trip_load(string)))
        context._set_raw_data(rd)
        assert context.channels == ('conda-forge',)

    def test_expandvars(self):
        """
        Environment variables should be expanded in settings that have expandvars=True.
        """
        def _get_expandvars_context(attr, config_expr, env_value):
            with mock.patch.dict(os.environ, {"TEST_VAR": env_value}):
                reset_context(())
                string = f"{attr}: {config_expr}"
                rd = odict(testdata=YamlRawParameter.make_raw_parameters('testdata', yaml_round_trip_load(string)))
                context._set_raw_data(rd)
                return getattr(context, attr)

        ssl_verify = _get_expandvars_context("ssl_verify", "${TEST_VAR}", "yes")
        assert ssl_verify

        for attr, env_value in [
            ("client_ssl_cert", "foo"),
            ("client_ssl_cert_key", "foo"),
            ("channel_alias", "http://foo"),
        ]:
            value = _get_expandvars_context(attr, "${TEST_VAR}", env_value)
            assert value == env_value

        for attr in [
            "migrated_custom_channels",
            "proxy_servers",
        ]:
            value = _get_expandvars_context("proxy_servers", "{'x': '${TEST_VAR}'}", "foo")
            assert value == {"x": "foo"}

        for attr in [
            "channels",
            "default_channels",
            "whitelist_channels",
        ]:
            value = _get_expandvars_context(attr, "['${TEST_VAR}']", "foo")
            assert value == ("foo",)

        custom_channels = _get_expandvars_context("custom_channels", "{'x': '${TEST_VAR}'}", "http://foo")
        assert custom_channels["x"].location == "foo"

        custom_multichannels = _get_expandvars_context("custom_multichannels", "{'x': ['${TEST_VAR}']}", "http://foo")
        assert len(custom_multichannels["x"]) == 1
        assert custom_multichannels["x"][0].location == "foo"

        envs_dirs = _get_expandvars_context("envs_dirs", "['${TEST_VAR}']", "/foo")
        assert any("foo" in d for d in envs_dirs)

        pkgs_dirs = _get_expandvars_context("pkgs_dirs", "['${TEST_VAR}']", "/foo")
        assert any("foo" in d for d in pkgs_dirs)


class ContextDefaultRcTests(TestCase):

    def test_subdirs(self):
        assert context.subdirs == (context.subdir, 'noarch')

        subdirs = ('linux-highest', 'linux-64', 'noarch')
        with env_var('CONDA_SUBDIRS', ','.join(subdirs), stack_callback=conda_tests_ctxt_mgmt_def_pol):
            assert context.subdirs == subdirs

    def test_local_build_root_default_rc(self):
        if context.root_writable:
            assert context.local_build_root == join(context.root_prefix, 'conda-bld')
        else:
            assert context.local_build_root == expand('~/conda-bld')



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import absolute_import, division, print_function, unicode_literals

from os.path import join
from tempfile import gettempdir

from conda.base.constants import UNKNOWN_CHANNEL

from conda.base.context import context, conda_tests_ctxt_mgmt_def_pol
from conda.common.io import env_var
from conda.common.url import join_url, path_to_url
from conda.gateways.disk.create import mkdir_p
from conda.gateways.disk.delete import rm_rf
from conda.models.dist import Dist
from logging import getLogger
from unittest import TestCase

import pytest

log = getLogger(__name__)


@pytest.mark.parametrize("fmt", [".conda", ".tar.bz2"])
def test_dist(fmt):
    d = Dist.from_string("spyder-app-2.3.8-py27_0{}".format(fmt))
    assert d.channel == UNKNOWN_CHANNEL
    assert d.quad[0] == "spyder-app"
    assert d.quad[1] == "2.3.8"
    assert d.quad[2] == "py27_0"
    assert d.build_number == 0
    assert d.dist_name == "spyder-app-2.3.8-py27_0"
    assert d.fmt == fmt

    assert d == Dist.from_string("spyder-app-2.3.8-py27_0")
    assert d != Dist.from_string("spyder-app-2.3.8-py27_1{}".format(fmt))

    d2 = Dist("spyder-app-2.3.8-py27_0{}".format(fmt))
    assert d == d2

    d3 = Dist(d2)
    assert d3 is d2

@pytest.mark.parametrize("fmt", [".conda", ".tar.bz2"])
def test_channel(fmt):
    d = Dist.from_string("conda-forge::spyder-app-2.3.8-py27_0{}".format(fmt))
    assert d.channel == 'conda-forge'
    assert d.quad[0] == "spyder-app"
    assert d.dist_name == "spyder-app-2.3.8-py27_0"
    assert d.fmt == fmt

    d = Dist.from_string("s3://some/bucket/name::spyder-app-2.3.8-py27_0{}".format(fmt))
    assert d.channel == 's3://some/bucket/name'
    assert d.quad[0] == "spyder-app"
    assert d.dist_name == "spyder-app-2.3.8-py27_0"
    assert d.to_url() == join_url("s3://some/bucket/name", context.subdir,
                                    "spyder-app-2.3.8-py27_0{}".format(fmt))


@pytest.mark.parametrize("fmt", [".conda", ".tar.bz2"])
def test_dist_with_channel_url(fmt):
    # standard named channel
    url = "https://repo.anaconda.com/pkgs/main/win-64/spyder-app-2.3.8-py27_0{}".format(fmt)
    d = Dist(url)
    assert d.channel == 'defaults'
    assert d.name == 'spyder-app'
    assert d.version == '2.3.8'
    assert d.build_string == 'py27_0'
    assert d.fmt == fmt

    assert d.to_url() == url
    assert d.is_channel is True

    # standard url channel
    url = "https://not.real.continuum.io/pkgs/main/win-64/spyder-app-2.3.8-py27_0{}".format(fmt)
    d = Dist(url)
    assert d.channel == 'defaults'  # because pkgs/anaconda is in defaults
    assert d.name == 'spyder-app'
    assert d.version == '2.3.8'
    assert d.build_string == 'py27_0'
    assert d.fmt == fmt

    assert d.to_url() == url
    assert d.is_channel is True

    # another standard url channel
    url = "https://not.real.continuum.io/not/anaconda/win-64/spyder-app-2.3.8-py27_0{}".format(fmt)
    d = Dist(url)
    assert d.channel == 'https://not.real.continuum.io/not/anaconda'
    assert d.name == 'spyder-app'
    assert d.version == '2.3.8'
    assert d.build_string == 'py27_0'
    assert d.fmt == fmt

    assert d.to_url() == url
    assert d.is_channel is True

    # local file url that is a named channel
    conda_bld_path = join(gettempdir(), 'conda-bld')
    try:
        mkdir_p(conda_bld_path)
        with env_var('CONDA_BLD_PATH', conda_bld_path, stack_callback=conda_tests_ctxt_mgmt_def_pol):
            url = path_to_url(join_url(context.croot, 'osx-64', 'bcrypt-3.1.1-py35_2{}'.format(fmt)))
            d = Dist(url)
            assert d.channel == 'local'
            assert d.name == 'bcrypt'
            assert d.version == '3.1.1'
            assert d.build_string == 'py35_2'
            assert d.fmt == fmt

            assert d.to_url() == url
            assert d.is_channel is True
    finally:
        rm_rf(conda_bld_path)

    # local file url that is not a named channel
    url = join_url('file:///some/location/on/disk', 'osx-64', 'bcrypt-3.1.1-py35_2{}'.format(fmt))
    d = Dist(url)
    assert d.channel == 'file:///some/location/on/disk'
    assert d.name == 'bcrypt'
    assert d.version == '3.1.1'
    assert d.build_string == 'py35_2'
    assert d.fmt == fmt

    assert d.to_url() == url
    assert d.is_channel is True



@pytest.mark.parametrize("fmt", [".conda", ".tar.bz2"])
def test_dist_with_non_channel_url(fmt):
    # contrived url
    url = "https://repo.anaconda.com/pkgs/anaconda/cffi-1.9.1-py34_0{}".format(fmt)
    d = Dist(url)
    assert d.channel == '<unknown>'
    assert d.name == 'cffi'
    assert d.version == '1.9.1'
    assert d.build_string == 'py34_0'
    assert d.fmt == fmt

    assert d.to_url() == url
    assert d.is_channel is False

    # file url that is not a channel
    url = path_to_url(join_url(context.croot, 'cffi-1.9.1-py34_0{}'.format(fmt)))
    d = Dist(url)
    assert d.channel == '<unknown>'
    assert d.name == 'cffi'
    assert d.version == '1.9.1'
    assert d.build_string == 'py34_0'
    assert d.fmt == fmt

    assert d.to_url() == url
    assert d.is_channel is False

    # file url that is a package cache
    # TODO: maybe this should look up the channel in urls.txt?  or maybe that's too coupled?
    url = join_url(path_to_url(context.pkgs_dirs[0]), 'cffi-1.9.1-py34_0{}'.format(fmt))
    d = Dist(url)
    assert d.channel == '<unknown>'
    assert d.name == 'cffi'
    assert d.version == '1.9.1'
    assert d.build_string == 'py34_0'
    assert d.fmt == fmt

    assert d.to_url() == url
    assert d.is_channel is False



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import absolute_import, division, print_function, unicode_literals

from distutils.sysconfig import get_python_lib
from logging import getLogger
import ntpath
import os
from os.path import abspath, dirname, isfile, join, realpath
import sys
from unittest import TestCase

import pytest

from conda import CONDA_PACKAGE_ROOT, CONDA_SOURCE_ROOT
from conda.auxlib.ish import dals
from conda.base.context import context, reset_context, conda_tests_ctxt_mgmt_def_pol
from conda.cli.common import stdout_json
from conda.common.compat import on_win, open
from conda.common.io import captured, env_var, env_vars
from conda.common.path import get_python_short_path, win_path_backout, win_path_ok
from conda.core.initialize import Result, _get_python_info, init_sh_system, init_sh_user, \
    initialize_dev, install, install_conda_csh, install_conda_fish, \
    install_conda_sh, install_conda_xsh, make_entry_point, make_entry_point_exe, \
    make_initialize_plan, make_install_plan, install_condabin_conda_bat
from conda.exceptions import CondaValueError
from conda.gateways.disk.create import create_link, mkdir_p
from conda.models.enums import LinkType
from conda.testing.helpers import tempdir

try:
    from unittest.mock import Mock, patch
except ImportError:
    from mock import Mock, patch


log = getLogger(__name__)


class InitializeTests(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.orig_verbosity = os.environ.get('CONDA_VERBOSITY')
        os.environ['CONDA_VERBOSITY'] = '1'
        reset_context(())

    @classmethod
    def tearDownClass(cls):
        if cls.orig_verbosity is None:
            del os.environ['CONDA_VERBOSITY']
        else:
            os.environ['CONDA_VERBOSITY'] = cls.orig_verbosity

    def test_get_python_info(self):
        python_exe, python_version, site_packages_dir = _get_python_info(sys.prefix)
        assert realpath(python_exe) == realpath(sys.executable)
        assert python_version == "%s.%s.%s" % sys.version_info[:3]
        assert site_packages_dir == get_python_lib()

    def test_make_install_plan(self):
        python_exe = "/darwin/bin/python"
        python_version = "2.6.10"
        site_packages_dir = "/darwin/lib/python2.6/site-packages"

        with patch("conda.core.initialize._get_python_info", return_value=(
                python_exe, python_version, site_packages_dir
        )):
            plan = make_install_plan("/darwin")
            stdout_json(plan)
            if on_win:
                assert plan == [
                    {
                        "function": "make_entry_point_exe",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\Scripts\\conda.exe"
                        }
                    },
                    {
                        "function": "make_entry_point_exe",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\Scripts\\conda-env.exe"
                        }
                    },
                    {
                        "function": "make_entry_point",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "func": "main",
                            "module": "conda.cli",
                            "target_path": "/darwin\\Scripts\\conda-script.py"
                        }
                    },
                    {
                        "function": "make_entry_point",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "func": "main",
                            "module": "conda_env.cli.main",
                            "target_path": "/darwin\\Scripts\\conda-env-script.py"
                        }
                    },
                    {
                        "function": "install_condabin_conda_bat",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\condabin\\conda.bat"
                        }
                    },
                    {
                        'function': 'install_library_bin_conda_bat',
                        'kwargs': {
                            'conda_prefix': '/darwin',
                            'target_path': '/darwin\\Library\\bin\\conda.bat'
                        }
                    },
                    {
                        "function": "install_condabin_conda_activate_bat",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\condabin\\_conda_activate.bat"
                        }
                    },
                    {
                        'function': 'install_condabin_rename_tmp_bat',
                        'kwargs': {'target_path': '/darwin\\condabin\\rename_tmp.bat',
                                   'conda_prefix': '/darwin'
                        }
                    },
                    {
                        "function": "install_condabin_conda_auto_activate_bat",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\condabin\\conda_auto_activate.bat"
                        }
                    },
                    {
                        "function": "install_condabin_hook_bat",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\condabin\\conda_hook.bat"
                        }
                    },
                    {
                        "function": "install_Scripts_activate_bat",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\Scripts\\activate.bat"
                        }
                    },
                    {
                        "function": "install_activate_bat",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\condabin\\activate.bat"
                        }
                    },
                    {
                        "function": "install_deactivate_bat",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\condabin\\deactivate.bat"
                        }
                    },
                    {
                        "function": "install_activate",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\Scripts\\activate"
                        }
                    },
                    {
                        "function": "install_deactivate",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\Scripts\\deactivate"
                        }
                    },
                    {
                        "function": "install_conda_sh",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\etc\\profile.d\\conda.sh"
                        }
                    },
                    {
                        "function": "install_conda_fish",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\etc\\fish\\conf.d\\conda.fish"
                        }
                    },
                    {
                        'function': 'install_conda_psm1',
                        'kwargs': {
                            'target_path': '/darwin\\shell\\condabin\\Conda.psm1',
                            'conda_prefix': '/darwin'
                        }
                    },
                    {
                        'function': 'install_conda_hook_ps1',
                        'kwargs': {
                            'target_path': '/darwin\\shell\\condabin\\conda-hook.ps1',
                            'conda_prefix': '/darwin'
                        }
                    },
                    {
                        "function": "install_conda_xsh",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin/lib/python2.6/site-packages\\xontrib\\conda.xsh"
                        }
                    },
                    {
                        "function": "install_conda_csh",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin\\etc\\profile.d\\conda.csh"
                        }
                    }
                ]
            else:
                assert plan == [
                    {
                        "function": "make_entry_point",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "func": "main",
                            "module": "conda.cli",
                            "target_path": "/darwin/condabin/conda"
                        }
                    },
                    {
                        "function": "make_entry_point",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "func": "main",
                            "module": "conda.cli",
                            "target_path": "/darwin/bin/conda"
                        }
                    },
                    {
                        "function": "make_entry_point",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "func": "main",
                            "module": "conda_env.cli.main",
                            "target_path": "/darwin/bin/conda-env"
                        }
                    },
                    {
                        "function": "install_activate",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin/bin/activate"
                        }
                    },
                    {
                        "function": "install_deactivate",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin/bin/deactivate"
                        }
                    },
                    {
                        "function": "install_conda_sh",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin/etc/profile.d/conda.sh"
                        }
                    },
                    {
                        "function": "install_conda_fish",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin/etc/fish/conf.d/conda.fish"
                        }
                    },
                    {
                        'function': 'install_conda_psm1',
                        'kwargs': {
                            'target_path': '/darwin/shell/condabin/Conda.psm1',
                            'conda_prefix': '/darwin'
                        }
                    },
                    {
                        'function': 'install_conda_hook_ps1',
                        'kwargs': {
                            'target_path': '/darwin/shell/condabin/conda-hook.ps1',
                            'conda_prefix': '/darwin'
                        }
                    },
                    {
                        "function": "install_conda_xsh",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin/lib/python2.6/site-packages/xontrib/conda.xsh"
                        }
                    },
                    {
                        "function": "install_conda_csh",
                        "kwargs": {
                            "conda_prefix": "/darwin",
                            "target_path": "/darwin/etc/profile.d/conda.csh"
                        }
                    }
                ]

    def test_make_initialize_plan_bash_zsh(self):
        with tempdir() as conda_temp_prefix:
            plan = make_initialize_plan(conda_temp_prefix, ('bash', 'zsh'), for_user=True,
                                        for_system=True, anaconda_prompt=False)
            steps = tuple(step for step in plan if step['function'] == 'init_sh_user')
            assert len(steps) == 2
            steps = tuple(step for step in plan if step['function'] == 'init_sh_system')
            assert len(steps) == 1

    def test_make_initialize_plan_cmd_exe(self):
        with tempdir() as conda_temp_prefix:
            plan = make_initialize_plan(conda_temp_prefix, ('cmd.exe',), for_user=True,
                                        for_system=True, anaconda_prompt=True)
            steps = tuple(step for step in plan if step['function'] == 'init_cmd_exe_registry')
            assert len(steps) == 2
            steps = tuple(step for step in plan if step['function'] == 'install_anaconda_prompt')
            assert len(steps) == 2
            steps = tuple(step for step in plan if step['function'] == 'init_long_path')
            assert len(steps) == 1

    def test_make_entry_point(self):
        with tempdir() as conda_temp_prefix:
            conda_prefix = abspath(sys.prefix)
            if on_win:
                conda_exe_path = join(conda_temp_prefix, 'Scripts', 'conda-script.py')
            else:
                conda_exe_path = join(conda_temp_prefix, 'bin', 'conda')
            result = make_entry_point(conda_exe_path, conda_prefix, 'conda.entry.point', 'run')
            assert result == Result.MODIFIED

            with open(conda_exe_path) as fh:
                ep_contents = fh.read()

            if on_win:
                assert ep_contents == dals("""
                # -*- coding: utf-8 -*-
                import sys

                if __name__ == '__main__':
                    from conda.entry.point import run
                    sys.exit(run())
                """)
            else:
                assert ep_contents == dals("""
                #!%s/bin/python
                # -*- coding: utf-8 -*-
                import sys

                if __name__ == '__main__':
                    from conda.entry.point import run
                    sys.exit(run())
                """) % conda_prefix

            result = make_entry_point(conda_exe_path, conda_prefix, 'conda.entry.point', 'run')
            assert result == Result.NO_CHANGE

    def test_make_entry_point_exe(self):
        with tempdir() as conda_temp_prefix:
            conda_prefix = abspath(sys.prefix)
            target_path = join(conda_temp_prefix, 'Scripts', 'conda-env.exe')
            result = make_entry_point_exe(target_path, conda_prefix)
            assert result == Result.MODIFIED

            assert isfile(target_path)

            result = make_entry_point_exe(target_path, conda_prefix)
            assert result == Result.NO_CHANGE

    def test_install_conda_sh(self):
        with tempdir() as conda_prefix:
            target_path = join(conda_prefix, 'etc', 'profile.d', 'conda.sh')
            context.dev = False
            result = install_conda_sh(target_path, conda_prefix)
            assert result == Result.MODIFIED

            with open(target_path) as fh:
                created_file_contents = fh.read()

            from conda.activate import PosixActivator
            activator = PosixActivator()

            line0, line1, line2, line3, _, remainder = created_file_contents.split('\n', 5)
            assert line0 == "export CONDA_EXE='%s'" % activator.path_conversion(context.conda_exe)
            assert line1 == "export _CE_M=''"
            assert line2 == "export _CE_CONDA=''"
            assert line3.startswith("export CONDA_PYTHON_EXE=")

            with open(join(CONDA_PACKAGE_ROOT, 'shell', 'etc', 'profile.d', 'conda.sh')) as fh:
                original_contents = fh.read()
            assert remainder == original_contents

            result = install_conda_sh(target_path, conda_prefix)
            assert result == Result.NO_CHANGE

    def test_install_conda_fish(self):
        with tempdir() as conda_temp_prefix:
            conda_prefix = abspath(sys.prefix)
            target_path = join(conda_temp_prefix, 'etc', 'fish', 'conf.d', 'conda.fish')
            result = install_conda_fish(target_path, conda_prefix)
            assert result == Result.MODIFIED

            with open(target_path) as fh:
                created_file_contents = fh.read()

            first_line, second_line, third_line, fourth_line, remainder = created_file_contents.split('\n', 4)
            if on_win:
                win_conda_exe = join(conda_prefix, 'Scripts', 'conda.exe')
                win_py_exe = join(conda_prefix, 'python.exe')
                assert first_line == 'set -gx CONDA_EXE (cygpath "%s")' % win_conda_exe
                assert second_line == 'set _CONDA_ROOT (cygpath "%s")' % conda_prefix
                assert third_line == 'set _CONDA_EXE (cygpath "%s")' % win_conda_exe
                assert fourth_line == 'set -gx CONDA_PYTHON_EXE (cygpath "%s")' % win_py_exe
            else:
                assert first_line == 'set -gx CONDA_EXE "%s"' % join(conda_prefix, 'bin', 'conda')
                assert second_line == 'set _CONDA_ROOT "%s"' % conda_prefix
                assert third_line == 'set _CONDA_EXE "%s"' % join(conda_prefix, 'bin', 'conda')
                assert fourth_line == 'set -gx CONDA_PYTHON_EXE "%s"' % join(conda_prefix, 'bin', 'python')

            with open(join(CONDA_PACKAGE_ROOT, 'shell', 'etc', 'fish', 'conf.d', 'conda.fish')) as fh:
                original_contents = fh.read()
            assert remainder == original_contents

            result = install_conda_fish(target_path, conda_prefix)
            assert result == Result.NO_CHANGE

    def test_install_conda_xsh(self):
        from conda.activate import XonshActivator

        with tempdir() as conda_temp_prefix:
            conda_prefix = abspath(sys.prefix)
            target_path = join(conda_temp_prefix, 'Lib', 'site-packages', 'conda.xsh')
            result = install_conda_xsh(target_path, conda_prefix)
            assert result == Result.MODIFIED

            with open(target_path) as fh:
                created_file_contents = fh.read()

            first_line, remainder = created_file_contents.split('\n', 1)
            if on_win:
                assert first_line == '$CONDA_EXE = "%s"' % XonshActivator.path_conversion(join(conda_prefix, 'Scripts', 'conda.exe'))
            else:
                assert first_line == '$CONDA_EXE = "%s"' % join(conda_prefix, 'bin', 'conda')

            with open(join(CONDA_PACKAGE_ROOT, 'shell', 'conda.xsh')) as fh:
                original_contents = fh.read()
            assert remainder == original_contents

            result = install_conda_xsh(target_path, conda_prefix)
            assert result == Result.NO_CHANGE

    def test_install_conda_csh(self):
        with tempdir() as conda_temp_prefix:
            conda_prefix = abspath(sys.prefix)
            target_path = join(conda_temp_prefix, 'etc', 'profile.d', 'conda.csh')
            result = install_conda_csh(target_path, conda_prefix)
            assert result == Result.MODIFIED

            with open(target_path) as fh:
                created_file_contents = fh.read()

            first_line, second_line, third_line, fourth_line, remainder = created_file_contents.split('\n', 4)
            if on_win:
                assert first_line == 'setenv CONDA_EXE `cygpath %s`' % join(conda_prefix, 'Scripts', 'conda.exe')
                assert second_line == 'setenv _CONDA_ROOT `cygpath %s`' % conda_prefix
                assert third_line == 'setenv _CONDA_EXE `cygpath %s`' % join(conda_prefix, 'Scripts', 'conda.exe')
                assert fourth_line == 'setenv CONDA_PYTHON_EXE `cygpath %s`' % join(conda_prefix, 'python.exe')
            else:
                assert first_line == 'setenv CONDA_EXE "%s"' % join(conda_prefix, 'bin', 'conda')
                assert second_line == 'setenv _CONDA_ROOT "%s"' % conda_prefix
                assert third_line == 'setenv _CONDA_EXE "%s"' % join(conda_prefix, 'bin', 'conda')
                assert fourth_line == 'setenv CONDA_PYTHON_EXE "%s"' % join(conda_prefix, 'bin', 'python')

            with open(join(CONDA_PACKAGE_ROOT, 'shell', 'etc', 'profile.d', 'conda.csh')) as fh:
                original_contents = fh.read()
            assert remainder == original_contents

            result = install_conda_csh(target_path, conda_prefix)
            assert result == Result.NO_CHANGE

    def test_install_condabin_conda_bat(self):
        with tempdir() as conda_temp_prefix:
            conda_prefix = abspath(sys.prefix)
            target_path = join(conda_temp_prefix, 'condabin', 'conda.bat')
            result = install_condabin_conda_bat(target_path, conda_prefix)
            assert result == Result.MODIFIED

            with open(target_path) as fh:
                created_file_contents = fh.read()

            remainder = created_file_contents

            with open(join(CONDA_PACKAGE_ROOT, 'shell', 'condabin', 'conda.bat')) as fh:
                original_contents = fh.read()
            assert remainder == original_contents

            result = install_condabin_conda_bat(target_path, conda_prefix)
            assert result == Result.NO_CHANGE

    def test__get_python_info(self):
        python_exe, python_version, site_packages_dir = _get_python_info(sys.prefix)
        assert python_exe == sys.executable
        assert python_version == '%d.%d.%d' % sys.version_info[:3]
        assert site_packages_dir.endswith('site-packages')

    def test_install_1(self):
        with env_vars({'CONDA_DRY_RUN': 'true', 'CONDA_VERBOSITY': '0'}, stack_callback=conda_tests_ctxt_mgmt_def_pol):
            with tempdir() as conda_temp_prefix:
                with captured() as c:
                    install(conda_temp_prefix)

        assert "WARNING: Cannot install xonsh wrapper" in c.stderr
        if on_win:
            modified_files = (
                'conda.exe',
                'conda-env.exe',
                'conda-script.py',
                'conda-env-script.py',
                'conda.bat',  # condabin/conda.bat
                'conda.bat',  # Library/bin/conda.bat
                '_conda_activate.bat',
                'rename_tmp.bat',
                'conda_auto_activate.bat',
                'conda_hook.bat',
                'activate.bat',
                'activate.bat',
                'deactivate.bat',
                'activate',
                'deactivate',
                'conda.sh',
                'conda.fish',
                'Conda.psm1',
                'conda-hook.ps1',
                'conda.csh',
            )
        else:
            modified_files = (
                'conda',  # condabin/conda
                'conda',  # bin/conda
                'conda-env',
                'activate',
                'deactivate',
                'conda.sh',
                'conda.fish',
                'Conda.psm1',
                'conda-hook.ps1',
                'conda.csh',
            )

        print(c.stdout)
        print(c.stderr, file=sys.stderr)

        assert c.stdout.count('modified') == len(modified_files)
        for fn in modified_files:
            line = next(line for line in c.stdout.splitlines() if line.strip().endswith(fn))
            assert line.strip().startswith('modified'), line

    def test_initialize_dev_bash(self):
        with pytest.raises(CondaValueError):
            initialize_dev('bash', conda_source_root=join('a', 'b', 'c'))

        with env_vars({'CONDA_DRY_RUN': 'true', 'CONDA_VERBOSITY': '0'}, stack_callback=conda_tests_ctxt_mgmt_def_pol):
            with tempdir() as conda_temp_prefix:
                new_py = abspath(join(conda_temp_prefix, get_python_short_path()))
                mkdir_p(dirname(new_py))
                create_link(abspath(sys.executable), new_py, LinkType.hardlink if on_win else LinkType.softlink)
                with captured() as c:
                    initialize_dev(
                        "bash",
                        dev_env_prefix=conda_temp_prefix,
                        conda_source_root=CONDA_SOURCE_ROOT,
                    )

        print(c.stdout)
        print(c.stderr, file=sys.stderr)

        if on_win:
            modified_files = (
                'conda.exe',
                'conda-env.exe',
                'conda-script.py',
                'conda-env-script.py',
                'conda.bat',  # condabin/conda.bat
                'conda.bat',  # Library/bin/conda.bat
                '_conda_activate.bat',
                'rename_tmp.bat',
                'conda_auto_activate.bat',
                'conda_hook.bat',
                'activate.bat',
                'activate.bat',
                'deactivate.bat',
                'activate',
                'deactivate',
                'conda.sh',
                'conda.fish',
                'Conda.psm1',
                'conda-hook.ps1',
                'conda.xsh',
                'conda.csh',
                'site-packages',  # remove conda in site-packages dir
                'conda.egg-link',
                'easy-install.pth',
                'conda.egg-info',
            )
        else:
            modified_files = (
                'conda',  # condabin/conda
                'conda',  # bin/conda
                'conda-env',
                'activate',
                'deactivate',
                'conda.sh',
                'conda.fish',
                'Conda.psm1',
                'conda-hook.ps1',
                'conda.xsh',
                'conda.csh',
                'site-packages',  # remove conda in site-packages dir
                'conda.egg-link',
                'easy-install.pth',
                'conda.egg-info',
            )

        stderr = c.stderr.replace('no change', 'modified')
        assert stderr.count('modified') == len(modified_files)

        for fn in modified_files:
            line = next(line for line in stderr.splitlines() if line.strip().endswith(fn))
            assert line.strip().startswith('modified'), line

        assert "unset CONDA_SHLVL" in c.stdout

    def test_initialize_dev_cmd_exe(self):
        with env_vars({'CONDA_DRY_RUN': 'true', 'CONDA_VERBOSITY': '0'}, stack_callback=conda_tests_ctxt_mgmt_def_pol):
            with tempdir() as conda_temp_prefix:
                new_py = abspath(join(conda_temp_prefix, get_python_short_path()))
                mkdir_p(dirname(new_py))
                create_link(abspath(sys.executable), new_py, LinkType.hardlink if on_win else LinkType.softlink)
                with captured() as c:
                    initialize_dev(
                        "cmd.exe",
                        dev_env_prefix=conda_temp_prefix,
                        conda_source_root=CONDA_SOURCE_ROOT,
                    )

        print(c.stdout)
        print(c.stderr, file=sys.stderr)

        if on_win:
            modified_files = (
                'conda.exe',
                'conda-env.exe',
                'conda-script.py',
                'conda-env-script.py',
                'conda.bat',  # condabin/conda.bat
                'conda.bat',  # Library/bin/conda.bat
                '_conda_activate.bat',
                'rename_tmp.bat',
                'conda_auto_activate.bat',
                'conda_hook.bat',
                'activate.bat',  # condabin/activate.bat
                'activate.bat',  # Scripts/activate.bat
                'deactivate.bat',
                'activate',
                'deactivate',
                'conda.sh',
                'conda.fish',
                'Conda.psm1',
                'conda-hook.ps1',
                'conda.xsh',
                'conda.csh',
                'site-packages',  # remove conda in site-packages dir
                'conda.egg-link',
                'easy-install.pth',
                'conda.egg-info',
            )
        else:
            modified_files = (
                'conda',  # condabin/conda
                'conda',  # bin/conda
                'conda-env',
                'activate',
                'deactivate',
                'conda.sh',
                'conda.fish',
                'Conda.psm1',
                'conda-hook.ps1',
                'conda.xsh',
                'conda.csh',
                'site-packages',  # remove conda in site-packages dir
                'conda.egg-link',
                'easy-install.pth',
                'conda.egg-info',

            )

        stderr = c.stderr.replace('no change', 'modified')
        assert stderr.count('modified') == len(modified_files)

        for fn in modified_files:
            line = next(line for line in stderr.splitlines() if line.strip().endswith(fn))
            assert line.strip().startswith('modified'), line

    @pytest.mark.skipif(on_win, reason="unix-only test")
    def test_init_sh_user_unix(self):
        with tempdir() as conda_temp_prefix:
            target_path = join(conda_temp_prefix, '.bashrc')

            initial_content = dals("""
            export PATH="/some/other/conda/bin:$PATH"
            export PATH="%(prefix)s/bin:$PATH"
              export PATH="%(prefix)s/bin:$PATH"

            # >>> conda initialize >>>
            __conda_setup="$('%(prefix)s/bin/conda' 'shell.bash' 'hook' 2> /dev/null)"
            if [ $? -eq 0 ]; then
            fi
            unset __conda_setup
            # <<< conda initialize <<<

            . etc/profile.d/conda.sh
            . etc/profile.d/coda.sh
            . /somewhere/etc/profile.d/conda.sh
            source /etc/profile.d/conda.sh

            \t source %(prefix)s/etc/profile.d/conda.sh
            """) % {
                'prefix': win_path_backout(abspath(conda_temp_prefix)),
            }

            with open(target_path, 'w') as fh:
                fh.write(initial_content)

            init_sh_user(target_path, conda_temp_prefix, 'bash')

            with open(target_path) as fh:
                new_content = fh.read()

            expected_new_content = dals("""
            export PATH="/some/other/conda/bin:$PATH"
            # export PATH="%(prefix)s/bin:$PATH"  # commented out by conda initialize
            # export PATH="%(prefix)s/bin:$PATH"  # commented out by conda initialize

            # >>> conda initialize >>>
            # !! Contents within this block are managed by 'conda init' !!
            __conda_setup="$('%(prefix)s/bin/conda' 'shell.bash' 'hook' 2> /dev/null)"
            if [ $? -eq 0 ]; then
                eval "$__conda_setup"
            else
                if [ -f "%(prefix)s/etc/profile.d/conda.sh" ]; then
                    . "%(prefix)s/etc/profile.d/conda.sh"
                else
                    export PATH="%(prefix)s/bin:$PATH"
                fi
            fi
            unset __conda_setup
            # <<< conda initialize <<<

            # . etc/profile.d/conda.sh  # commented out by conda initialize
            . etc/profile.d/coda.sh
            # . /somewhere/etc/profile.d/conda.sh  # commented out by conda initialize
            # source /etc/profile.d/conda.sh  # commented out by conda initialize

            # source %(prefix)s/etc/profile.d/conda.sh  # commented out by conda initialize
            """) % {
                'prefix': win_path_backout(abspath(conda_temp_prefix)),
            }
            print(new_content)
            assert new_content == expected_new_content

            expected_reversed_content = dals("""
            export PATH="/some/other/conda/bin:$PATH"
            export PATH="%(prefix)s/bin:$PATH"
            export PATH="%(prefix)s/bin:$PATH"

            . etc/profile.d/conda.sh
            . etc/profile.d/coda.sh
            . /somewhere/etc/profile.d/conda.sh
            source /etc/profile.d/conda.sh

            source %(prefix)s/etc/profile.d/conda.sh
            """) % {
                'prefix': win_path_backout(abspath(conda_temp_prefix)),
            }

            init_sh_user(target_path, conda_temp_prefix, 'bash', reverse=True)
            with open(target_path) as fh:
                reversed_content = fh.read()
            print(reversed_content)
            assert reversed_content == expected_reversed_content

    @pytest.mark.skipif(on_win, reason="unix-only test")
    def test_init_sh_user_tcsh_unix(self):
        with tempdir() as conda_temp_prefix:
            target_path = join(conda_temp_prefix, '.tcshrc')
            init_sh_user(target_path, conda_temp_prefix, 'tcsh')

            with open(target_path) as fh:
                tcshrc_contet = fh.read()

            conda_csh = "%s/etc/profile.d/conda.csh" % conda_temp_prefix
            # tcsh/csh doesn't give users the ability to register a function.
            # Make sure that conda doesn't try to register a function
            conda_eval_string = "eval \"$__conda_setup\""
            assert conda_csh in tcshrc_contet
            assert conda_eval_string not in tcshrc_contet

    @pytest.mark.skipif(not on_win, reason="windows-only test")
    def test_init_sh_user_windows(self):
        with tempdir() as conda_temp_prefix:
            target_path = join(conda_temp_prefix, '.bashrc')
            conda_prefix = "c:\\Users\\Lars\\miniconda"
            cygpath_conda_prefix = "/c/Users/Lars/miniconda"

            initial_content = dals("""
            source /c/conda/Scripts/activate root
            . $(cygpath 'c:\\conda\\Scripts\\activate') root

            # >>> conda initialize >>>
            __conda_setup="$('%(prefix)s/bin/conda' 'shell.bash' 'hook' 2> /dev/null)"
            if [ $? -eq 0 ]; then
            fi
            unset __conda_setup
            # <<< conda initialize <<<

            . etc/profile.d/conda.sh
            . etc/profile.d/coda.sh
            . /somewhere/etc/profile.d/conda.sh
            source /etc/profile.d/conda.sh

            \t source %(prefix)s/etc/profile.d/conda.sh
            """) % {
                'prefix': win_path_ok(abspath(conda_prefix)),
            }

            with open(target_path, 'w') as fh:
                fh.write(initial_content)

            init_sh_user(target_path, conda_prefix, 'bash')

            with open(target_path) as fh:
                new_content = fh.read()

            print(new_content)

            expected_new_content = dals("""
            # source /c/conda/Scripts/activate root  # commented out by conda initialize
            # . $(cygpath 'c:\\conda\\Scripts\\activate') root  # commented out by conda initialize

            # >>> conda initialize >>>
            # !! Contents within this block are managed by 'conda init' !!
            eval "$('%(cygpath_conda_prefix)s/Scripts/conda.exe' 'shell.bash' 'hook')"
            # <<< conda initialize <<<

            # . etc/profile.d/conda.sh  # commented out by conda initialize
            . etc/profile.d/coda.sh
            # . /somewhere/etc/profile.d/conda.sh  # commented out by conda initialize
            # source /etc/profile.d/conda.sh  # commented out by conda initialize

            # source %(prefix)s/etc/profile.d/conda.sh  # commented out by conda initialize
            """) % {
                'prefix': win_path_ok(abspath(conda_prefix)),
                'cygpath_conda_prefix': cygpath_conda_prefix,
            }

            assert new_content == expected_new_content

            expected_reversed_content = dals("""
            source /c/conda/Scripts/activate root
            . $(cygpath 'c:\\conda\\Scripts\\activate') root

            . etc/profile.d/conda.sh
            . etc/profile.d/coda.sh
            . /somewhere/etc/profile.d/conda.sh
            source /etc/profile.d/conda.sh

            source %(prefix)s/etc/profile.d/conda.sh
            """) % {
                'prefix': win_path_ok(abspath(conda_prefix)),
            }

            init_sh_user(target_path, conda_temp_prefix, 'bash', reverse=True)
            with open(target_path) as fh:
                reversed_content = fh.read()
            print(reversed_content)
            assert reversed_content == expected_reversed_content

    @pytest.mark.skipif(not on_win, reason="win-only test")
    def test_init_cmd_exe_registry(self):
        def _read_windows_registry_mock(target_path, value=None):
            if not value:
                value = "yada\\yada\\conda_hook.bat"
            return 'echo hello & if exist "{v}" "{v}" & echo "world"'.format(v=value), None

        from conda.core import initialize
        orig_read_windows_registry = initialize._read_windows_registry
        initialize._read_windows_registry = _read_windows_registry_mock
        orig_join = initialize.join
        initialize.join = ntpath.join

        try:
            target_path = r'HKEY_CURRENT_USER\Software\Microsoft\Command Processor\AutoRun'
            conda_prefix = "c:\\Users\\Lars\\miniconda"
            with env_var('CONDA_DRY_RUN', 'true', stack_callback=conda_tests_ctxt_mgmt_def_pol):
                with captured() as c:
                    initialize.init_cmd_exe_registry(target_path, conda_prefix)
        finally:
            initialize._read_windows_registry = orig_read_windows_registry
            initialize.join = orig_join

        expected = "echo hello & if exist \"c:\\Users\\Lars\\miniconda\\condabin\\conda_hook.bat\" \"c:\\Users\\Lars\\miniconda\\condabin\\conda_hook.bat\" & echo \"world\""
        assert c.stdout.strip().splitlines()[-1][1:] == expected

        # test the reverse (remove the key)
        initialize._read_windows_registry = lambda x: _read_windows_registry_mock(x, value=join(conda_prefix, 'condabin', 'conda_hook.bat'))
        initialize.join = ntpath.join
        try:
            target_path = r'HKEY_CURRENT_USER\Software\Microsoft\Command Processor\AutoRun'
            conda_prefix = "c:\\Users\\Lars\\miniconda"
            with env_var('CONDA_DRY_RUN', 'true', stack_callback=conda_tests_ctxt_mgmt_def_pol):
                with captured() as c:
                    initialize.init_cmd_exe_registry(target_path, conda_prefix, reverse=True)
        finally:
            initialize._read_windows_registry = orig_read_windows_registry
            initialize.join = orig_join

        expected = "echo hello & echo \"world\""
        assert c.stdout.strip().splitlines()[-1][1:] == expected


    @pytest.mark.skipif(not on_win, reason="win-only test")
    def test_init_enable_long_path(self):
        self.dummy_value = 0

        def _read_windows_registry_mock(target_path):
            return self.dummy_value, "REG_DWORD"

        def _write_windows_registry_mock(target_path, value, dtype):
            self.dummy_value = value

        from conda.core import initialize
        orig_read_windows_registry = initialize._read_windows_registry
        initialize._read_windows_registry = _read_windows_registry_mock
        orig_write_windows_registry = initialize._write_windows_registry
        initialize._write_windows_registry = _write_windows_registry_mock
        orig_join = initialize.join
        initialize.join = ntpath.join

        try:
            target_path = r'HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\FileSystem\\LongPathsEnabled'
            assert initialize._read_windows_registry(target_path)[0] == 0
            initialize.init_long_path(target_path)
            assert initialize._read_windows_registry(target_path)[0] == 1
        finally:
            initialize._read_windows_registry = orig_read_windows_registry
            initialize._write_windows_registry = orig_write_windows_registry
            initialize.join = orig_join

    def test_init_sh_system(self):
        with tempdir() as td:
            target_path = join(td, 'conda.sh')
            conda_prefix = join(td, 'b', 'c')
            init_sh_system(target_path, conda_prefix)
            with open(target_path) as fh:
                content = fh.read().strip().splitlines()
            assert content[0] == '# >>> conda initialize >>>'
            assert content[1] == "# !! Contents within this block are managed by 'conda init' !!"
            assert content[-1] == '# <<< conda initialize <<<'

            init_sh_system(target_path, conda_prefix, reverse=True)
            assert not isfile(target_path)



# -*- coding: utf-8 -*-
# Copyright (C) 2012 Anaconda, Inc
# SPDX-License-Identifier: BSD-3-Clause

import unittest
try:
    from unittest import mock
except ImportError:
    import mock

import os
from conda_env.installers import pip
from conda.exceptions import CondaValueError


class PipInstallerTest(unittest.TestCase):
    def test_straight_install(self):

        # To check that the correct file would be written
        written_deps = []

        def log_write(text):
            written_deps.append(text)
            return mock.DEFAULT

        with mock.patch.object(pip.subprocess, 'Popen') as mock_popen, \
                mock.patch.object(pip, 'pip_args') as mock_pip_args, \
                mock.patch('tempfile.NamedTemporaryFile', mock.mock_open()) as mock_namedtemp:
            # Mock
            mock_popen.return_value.returncode = 0
            mock_pip_args.return_value = (['pip'], '9.0.1')
            mock_namedtemp.return_value.write.side_effect = log_write
            mock_namedtemp.return_value.name = 'tmp-file'
            args = mock.Mock()
            root_dir = '/whatever' if os.name != 'nt' else 'C:\\whatever'
            args.file = os.path.join(root_dir, 'environment.yml')
            # Run
            pip.install('/some/prefix', ['foo', '-e ./bar'], args)
            # Check expectations
            mock_popen.assert_called_with(['pip', 'install', '-r', 'tmp-file'],
                                          cwd=root_dir,
                                          universal_newlines=True)
            self.assertEqual(1, mock_popen.return_value.communicate.call_count)
            self.assertEqual(written_deps, ['foo\n-e ./bar'])

    def test_stops_on_exception(self):
        with mock.patch.object(pip.subprocess, 'Popen') as popen:
            popen.return_value.returncode = 22
            with mock.patch.object(pip, 'pip_args') as pip_args:
                # make sure that installed doesn't bail early
                pip_args.return_value = (['pip'], '9.0.1')

                self.assertRaises(CondaValueError, pip.install,
                                  '/some/prefix', ['foo'], None)
